function Y = MSLocUknTxPosMvgObj_BRSDP(r,d,r_v,d_v,so,Q1,l,Y1)
% Y = MSLocUknTxPosMvgObj_BRSDP(r,d,r_v,d_v,so,Q1,l,Y1)
%
% This program realizes the BR-SDP solution for estimate the object position and velocity, the
% transmitter position, and the range and range rate offsets using the TD and DFS measurement.
%
% Usage: Y1 = MSLocUknTxPosMvgObj_BRSDP(r,d,r_v,d_v,s,Q1,1,Y);
%
% Input Parameter List:
% r:   Nx1 Indirect range measurement vector after narrow scene.
% d:   Nx1 Direct range measurement vector after narrow scene.
% r_v: Nx1 Indirect range rate measurement vector after narrow scene.
% d_v: Nx1 Direct range rate measurement vector after narrow scene.
% s:   KxN Receiver position matrix after narrow scene.
% Q1:  (4*N)x(4*N) Covariance matrix after narrow scene.
% l:   if l==1, solve the BR-SDP problem without constraint (29), if l==2, solve the BR-SDP problem include (29).
% Y1:  (3*K+8)x(3*K+8) The solution of SDP problem.
%
% Output Parameter List:
% Y:   (3*K+8)x(3*K+8) The solution of BR-SDP problem.
%
% Reference:
% J. Pei, G. Wang, K. C. Ho, and L. Huang, "Reducing bias for multistatic
% localization of a moving object by transmitter at unknown position," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 5, pp. 5324-5341, Oct. 2023.
% 
% Jian Pei, Gang Wang, K. C. Ho and Lei Huang	Nov. 2023
%
%       Copyright (C) 2023
%       Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China.
%       wanggang@nbu.edu.cn
%

[K,N] = size(so);

for j=1:N
    b1(j) = 0.5*(r(j)^2-norm(so(:,j))^2);
    b2(j) = 0.5*(d(j)^2-norm(so(:,j))^2);
    b3(j) = r(j)*r_v(j);
    b4(j) = d_v(j);
end
b = [b1,b2,b3,b4]'; % the matrix 'b' in (15)

A1 = [-so',zeros(N,2*K),r,zeros(N,1),r,zeros(N,1),-0.5*ones(N,1),zeros(N,2)];
A2 = [zeros(N,2*K),-so',d,zeros(N,4),-0.5*ones(N,1),zeros(N,1)];
A3 = [zeros(N,K),-so',zeros(N,K),r_v,r,r_v,r,zeros(N,2),-0.5*ones(N,1)];
A4 = [zeros(N,3*K+1),ones(N,1),zeros(N,5)];
A = [A1',A2',A3',A4']'; % the matrix 'A' in (15)

G = [-A,b]; %the matrix 'G' in (18a)
%====Omega====
u_hat = Y1(1:K,3*K+8)/Y1(3*K+8,3*K+8);
uv_hat = Y1(K+1:2*K,3*K+8)/Y1(3*K+8,3*K+8);
t_hat = Y1(2*K+1:3*K,3*K+8)/Y1(3*K+8,3*K+8);
delta_o = Y1(3*K+1,3*K+8)/Y1(3*K+8,3*K+8);
delta_fo = Y1(3*K+2,3*K+8)/Y1(3*K+8,3*K+8);

for j=1:N
    ro(j) = norm(u_hat - t_hat)+norm(u_hat - so(:,j))+delta_o;
    do(j) = norm(t_hat - so(:,j))+delta_o;
    ro_v(j) = ((u_hat - t_hat)'/norm(u_hat - t_hat) + ...
        (u_hat - so(:,j))'/norm(u_hat - so(:,j)))*uv_hat+delta_fo;
    do_v(j) = delta_fo;
end

for j=1:N
    b1_star(j) = 0.5*(ro(j)^2-norm(so(:,j))^2);
    b2_star(j) = 0.5*(do(j)^2-norm(so(:,j))^2);
    b3_star(j) = ro(j)*ro_v(j);
    b4_star(j) = do_v(j);
end
b_star = [b1_star,b2_star,b3_star,b4_star]';

A1_star = [-so',zeros(N,2*K),ro',zeros(N,1),ro',zeros(N,1),-0.5*ones(N,1),zeros(N,2)];
A2_star = [zeros(N,2*K),-so',do',zeros(N,4),-0.5*ones(N,1),zeros(N,1)];
A3_star = [zeros(N,K),-so',zeros(N,K),ro_v',ro',ro_v',ro',zeros(N,2),-0.5*ones(N,1)];
A4_star = [zeros(N,3*K+1),ones(N,1),zeros(N,5)];
A_star = [A1_star',A2_star',A3_star',A4_star']';

G_star = [-A_star,b_star]; %the matrix 'G^o' in (63)

for j=1:N
    b11(j) = norm(u_hat - so(:,j));
    b21(j) = norm(t_hat - so(:,j));
    b31(j) = ((u_hat - so(:,j))'/norm(u_hat - so(:,j)))*uv_hat;
    b41(j) = 1;
end
B1 = diag(b11);
B2 = diag(b21);
B3 = diag(b31);
B4 = diag(b41);

B = [blkdiag(B1,B2),zeros(2*N,2*N);
    blkdiag(B3,zeros(N,N)),blkdiag(B1,B4)];

W = (inv(B*Q1*B'));

g1 = [ro,do]';
g2 = [ro_v,zeros(1,N)]';
g3 = [ro,ones(1,N)]';

H1 = [zeros(N,3*K),ones(N,1),zeros(N,1),ones(N,1),zeros(N,4);
    zeros(N,3*K),ones(N,1),zeros(N,6)];
H2 = [zeros(N,3*K),ones(N,1),zeros(N,1),ones(N,1),zeros(N,4);
    zeros(N,3*K+7)];
H3 = [zeros(N,3*K+1),ones(N,1),zeros(N,1),ones(N,1),zeros(N,3);
    zeros(N,3*K+7)]; % "H1,H2,H3,g1^o,g2^o,g3^o" in (57)

for j=1:N
    datau(j,:) = (u_hat-so(:,j))'/norm(u_hat-so(:,j));
    datat(j,:) = (t_hat-so(:,j))'/norm(t_hat-so(:,j));
end
dataut = (u_hat-t_hat)'/norm(u_hat-t_hat);
for j=1:N
    J1_1(j,(1+(j-1)*K):(K*j)) = datau(j,:);
    J1_2(j,(1+(j-1)*K):(K*j)) = datat(j,:);
    J3_1(j,(1+(j-1)*K):(K*j)) = uv_hat'/norm(u_hat-so(:,j))- ...
        uv_hat'*(u_hat-so(:,j))/(norm(u_hat-so(:,j)))^2*datau(j,:);
end
J1 = blkdiag(J1_1,J1_2);
J2 = blkdiag(J1_1,zeros(N,3*N));
J3 = blkdiag(J3_1,zeros(N,3*N)); % "J1^o,J2^o,J3^o" in (60)

for j=1:N
    V1(j,:) = (t_hat-so(:,j))'+(ro(j)-delta_o)*dataut;
    V2(j,:) = (u_hat-t_hat)'-(ro(j)-delta_o)*dataut;
    V3(j,:) = (ro(j)-delta_o)-norm(u_hat-t_hat);
    V4(j,:) = (t_hat-so(:,j))';
    V5(j,:) = do(j)-delta_o;
    V6(j,:) = (ro(j)-delta_o)*(uv_hat'/norm(u_hat-t_hat)-(u_hat-t_hat)'*uv_hat/(norm(u_hat-t_hat))^2*dataut)+(ro_v(j)-delta_fo)*dataut;
    V7(j,:) = (ro(j)-delta_o)*dataut+(t_hat-so(:,j))';
    V8(j,:) = uv_hat'+(delta_fo-ro_v(j))*dataut+(delta_o-ro(j))*(uv_hat'/norm(u_hat-t_hat)-(u_hat-t_hat)'*uv_hat/(norm(u_hat-t_hat))^2*dataut);
    V9(j,:) = (ro_v(j)-delta_fo)-dataut*uv_hat;
    V10(j,:) = (ro(j)-delta_o)-norm(u_hat-t_hat);
    V11(j,:) = 1;
end
V = [V1,zeros(N,K),V2,V3,zeros(N,1);
    zeros(N,2*K),V4,V5,zeros(N,1);
    V6,V7,V8,V9,V10;
    zeros(N,3*K+1),V11];

L = (inv(V'*W*V))*V'*W*B;

D1 = blkdiag(kron(eye(N),L(1:K,:)),kron(eye(N),L(2*K+1:3*K,:)));
D2 = blkdiag(kron(eye(N),L(1:K,:)),zeros(K*N,4*N*N));
D3 = blkdiag(kron(eye(N),L(K+1:2*K,:)),zeros(K*N,4*N*N)); % "D1^o,D2^o,D3^o" in (62)

Omega11 = [-H1,g1]'*(Q1(1:2*N,1:2*N).*W(1:2*N,1:2*N))*[-H1,g1];
Omega12 = [-H3,g2]'*(Q1(1:2*N,1:2*N).*W(2*N+1:4*N,1:2*N))*[-H1,g1];
Omega13 = [-H1,g1]'*(Q1(1:2*N,1:2*N).*W(1:2*N,2*N+1:4*N))*[-H3,g2];
Omega14 = [-H3,g2]'*(Q1(1:2*N,1:2*N).*W(2*N+1:4*N,2*N+1:4*N))*[-H3,g2]+ ...
    [-H2,g3]'*(Q1(2*N+1:4*N,2*N+1:4*N).*W(2*N+1:4*N,2*N+1:4*N))*[-H2,g3];
Omega1 = Omega11+Omega12+Omega13+Omega14;% "Omega1" in (63)

inv_b = inv(B');
Omega21 = [-H1,g1]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,1:2*N),ones(1,4*N))))*D1'*J1';
Omega22 = [-H3,g2]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*D1'*J1'+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q1(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*D1'*J1';
Omega23 = [-H1,g1]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2');
Omega24 = [-H3,g2]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2')+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q1(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2');
Omega25 = [-H1,g1]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2';
Omega26 = [-H3,g2]'*((kron(ones(1,2*N),Q1(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2'+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q1(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2';
Omega2 = [Omega21+Omega22,Omega23+Omega24+Omega25+Omega26]*W*G_star; % "Omega2" in (63)

inv_b1 = inv(B);
Omega31 = J1*D1*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega32 = J1*D1*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    J1*D1*((kron(ones(2*N,1),Q1(1:4*N,2*N+1:4*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega33 = (J2*D3+J3*D2)*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega34 = (J3*D2+J2*D3)*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    (J3*D2+J2*D3)*((kron(ones(2*N,1),Q1(1:4*N,2*N+1:4*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega35 = J2*D2*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(2*N+1:4*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega36 = J2*D2*((kron(ones(2*N,1),Q1(1:4*N,1:2*N))).*(kron(inv_b1(2*N+1:4*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    J2*D2*((kron(ones(2*N,1),Q1(1:4*N,2*N+1:4*N))).*(kron(inv_b1(2*N+1:4*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega3 = G_star'*W*[Omega31+Omega32;Omega33+Omega34+Omega35+Omega36]; % "Omega5" in (63)

a411 = W(1:2*N,1:2*N)*J1*D1;
a421 = W(2*N+1:4*N,1:2*N)*J1*D1;
a431 = W(1:2*N,2*N+1:4*N)*J2*D3;
a434 = W(1:2*N,2*N+1:4*N)*J3*D2;
a441 = W(2*N+1:4*N,2*N+1:4*N)*J2*D3;
a442 = W(2*N+1:4*N,2*N+1:4*N)*J3*D2;
a451 = W(1:2*N,2*N+1:4*N)*J2*D2;
a461 = W(2*N+1:4*N,2*N+1:4*N)*J2*D2;
for i=1:2*N
    a412(:,i) = diag(a411(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a422(:,i) = diag(a421(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a423(:,i) = diag(a421(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,2*N+1:4*N));
    a432(:,i) = diag(a431(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a433(:,i) = diag(a434(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a443(:,i) = diag(a441(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a444(:,i) = diag(a442(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a445(:,i) = diag(a441(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,2*N+1:4*N));
    a446(:,i) = diag(a442(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,2*N+1:4*N));
    a452(:,i) = diag(a451(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a462(:,i) = diag(a461(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,1:2*N));
    a463(:,i) = diag(a461(:,4*N*(i-1)+1:4*N*i)*Q1(1:4*N,2*N+1:4*N));
end
Omega41 = [-H1,g1]'*a412;
Omega42 = [-H3,g2]'*a422+[-H2,g3]'*a423;
Omega43 = [-H1,g1]'*(a432+a433);
Omega44 = [-H3,g2]'*(a443+a444)+[-H2,g3]'*(a445+a446);
Omega45 = [-H1,g1]'*a452;
Omega46 = [-H3,g2]'*a462+[-H2,g3]'*a463;
Omega4 = [Omega41+Omega42+Omega43+Omega44,Omega45+Omega46]*(inv(B))*G_star;% "Omega4" in (63)

a511 = D1'*J1'*W(1:2*N,1:2*N);
a521 = D3'*J2'*W(2*N+1:4*N,1:2*N);
a522 = D2'*J3'*W(2*N+1:4*N,1:2*N);
a531 = D1'*J1'*W(1:2*N,2*N+1:4*N);
a541 = D3'*J2'*W(2*N+1:4*N,2*N+1:4*N);
a542 = D2'*J3'*W(2*N+1:4*N,2*N+1:4*N);
a551 = D2'*J2'*W(2*N+1:4*N,1:2*N);
a561 = D2'*J2'*W(2*N+1:4*N,2*N+1:4*N);
for i=1:2*N
    a512(i,:) = diag(Q1(1:2*N,1:4*N)*a511(4*N*(i-1)+1:4*N*i,:));
    a523(i,:) = diag(Q1(1:2*N,1:4*N)*a521(4*N*(i-1)+1:4*N*i,:));
    a524(i,:) = diag(Q1(1:2*N,1:4*N)*a522(4*N*(i-1)+1:4*N*i,:));
    a532(i,:) = diag(Q1(1:2*N,1:4*N)*a531(4*N*(i-1)+1:4*N*i,:));
    a533(i,:) = diag(Q1(2*N+1:4*N,1:4*N)*a531(4*N*(i-1)+1:4*N*i,:));
    a543(i,:) = diag(Q1(1:2*N,1:4*N)*a541(4*N*(i-1)+1:4*N*i,:));
    a544(i,:) = diag(Q1(2*N+1:4*N,1:4*N)*a541(4*N*(i-1)+1:4*N*i,:));
    a545(i,:) = diag(Q1(1:2*N,1:4*N)*a542(4*N*(i-1)+1:4*N*i,:));
    a546(i,:) = diag(Q1(2*N+1:4*N,1:4*N)*a542(4*N*(i-1)+1:4*N*i,:));
    a552(i,:) = diag(Q1(1:2*N,1:4*N)*a551(4*N*(i-1)+1:4*N*i,:));
    a562(i,:) = diag(Q1(1:2*N,1:4*N)*a561(4*N*(i-1)+1:4*N*i,:));
    a563(i,:) = diag(Q1(2*N+1:4*N,1:4*N)*a561(4*N*(i-1)+1:4*N*i,:));
end

Omega51 = a512*[-H1,g1];
Omega52 = (a523+a524)*[-H1,g1];
Omega53 = a532*[-H3,g2]+a533*[-H2,g3];
Omega54 = (a543+a545)*[-H3,g2]+(a544+a546)*[-H2,g3];
Omega55 = a552*[-H1,g1];
Omega56 = a562*[-H3,g2]+a563*[-H2,g3];
Omega5 = G_star'*(inv(B'))*[Omega51+Omega52+Omega53+Omega54;Omega55+Omega56];% "Omega3" in (63)

Omega = Omega1-Omega2-Omega3-Omega4-Omega5;% "Omega" in (21)
k = 0.1;
%-----------------------1---------------------

for i=1
    cvx_clear
    cvx_begin sdp
    cvx_solver sdpt3
    %     cvx_solver sedumi
    cvx_precision best
    cvx_quiet(1)
    variable Y(3*K+8,3*K+8) symmetric
    minimize (trace(G'*W*G*Y))
    subject to
    Y(3*K+3,3*K+3) == trace(Y(1:K,1:K))+trace(Y(2*K+1:3*K,2*K+1:3*K))-2*trace(Y(1:K,(2*K+1:3*K)));
    Y(3*K+3,3*K+4) == trace(Y(1:K,K+1:2*K))-trace(Y(2*K+1:3*K,K+1:2*K));
    Y(3*K+5,3*K+8) == Y(3*K+1,3*K+1)+2*Y(3*K+1,3*K+3)-2*trace(Y(1:K,2*K+1:3*K))+trace(Y(2*K+1:3*K,2*K+1:3*K));
    Y(3*K+6,3*K+8) == Y(3*K+1,3*K+1)-trace(Y(2*K+1:3*K,2*K+1:3*K));
    Y(3*K+7,3*K+8) == 2*Y(3*K+1,3*K+2)+2*Y(3*K+1,3*K+4)+2*Y(3*K+2,3*K+3)-2*trace(Y(2*K+1:3*K,K+1:2*K));

    if l==2
        Y(3*K+4,3*K+4) <= trace(Y(K+1:2*K,K+1:2*K));
    end

    trace(Omega*Y) == k;

    Y >= 0;

    cvx_end

end
