function Z_star_BS=MSLocUknTxPosMvgObj_BsThy(uo,to,uo_v,delta_o,delta_fo,so,ro,do,ro_v,do_v,Q,crlb)
% function Z_star_MSLocUknTxPosMvgObj_BsThy(uo,to,uo_v,delta_o,delta_fo,so,ro,do,ro_v,do_v,Q,crlb)
%
% This program computes the theoretical bias of the BR-CWLS solution.
%
% Usage: theory_bias = MSLocUknTxPosMvgObj_BsThy(uo(:,iii),to(:,iii),uo_v,delta_o,delta_fo,so(:,1:N,iii),ro,do,ro_v,do_v,Q,mse(:,:,iii));
%
% Input Parameter List:
% uo:       Kxcj object position matrix.
% to:       (KxM)xcj transmitter position matrix.
% uo_v:     Kxcj object velocity matrix.
% delta_o:  1xcj  Range offset vector
% delta_fo: 1xcj  Range rate offset vector
% so:       (KxN)xcj receiver positions matrix.
%           N: Number of receivers.
% ro:       Nx1 true indirect range measurement vector
% do:       Nx1 true direct range measurement vector
% ro_v:     Nx1 true indirect range rate measurement vector
% do_v:     Nx1 true direct range rate measurement vector
% Q:        Covariance matrix
% mse:      CRLB matrix of the object position and velocity, the transmitter
%           position, and the range and range rate offsets estimate
%
% Output Parameter List:
% Z_star_BS:         bias in the final source location estimate
%
% Reference:
% J. Pei, G. Wang, K. C. Ho, and L. Huang, "Reducing bias for multistatic
% localization of a moving object by transmitter at unknown position," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 5, pp. 5324-5341, Oct. 2023.
% 
% Jian Pei, Gang Wang, K. C. Ho and Lei Huang	Nov. 1, 2023
%
%       Copyright (C) 2023
%       Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China.
%       wanggang@nbu.edu.cn
%

[K,N] = size(so);

for j=1:N
    b1(j) = 0.5*(ro(j)^2-norm(so(:,j))^2);
    b2(j) = 0.5*(do(j)^2-norm(so(:,j))^2);
    b3(j) = ro(j)*ro_v(j);
    b4(j) = do_v(j);
end
bo = [b1,b2,b3,b4]';

A1 = [-so',zeros(N,2*K),ro',zeros(N,1),ro',zeros(N,1),-0.5*ones(N,1),zeros(N,2)];
A2 = [zeros(N,2*K),-so',do',zeros(N,4),-0.5*ones(N,1),zeros(N,1)];
A3 = [zeros(N,K),-so',zeros(N,K),ro_v',ro',ro_v',ro',zeros(N,2),-0.5*ones(N,1)];
A4 = [zeros(N,3*K+1),ones(N,1),zeros(N,5)];
Ao = [A1',A2',A3',A4']';

G = [-Ao,bo];

for j=1:N
    b11(j) = norm(uo - so(:,j));
    b21(j) = norm(to - so(:,j));
    b31(j) = ((uo - so(:,j))'/norm(uo - so(:,j)))*uo_v;
    b41(j) = 1;
end
B1 = diag(b11);
B2 = diag(b21);
B3 = diag(b31);
B4 = diag(b41);

Bo = [blkdiag(B1,B2),zeros(2*N,2*N);
    blkdiag(B3,zeros(N,N)),blkdiag(B1,B4)];

Wo = (inv(Bo*Q*Bo'));

g1 = [ro,do]';
g2 = [ro_v,zeros(1,N)]';
g3 = [ro,ones(1,N)]';

H1 = [zeros(N,3*K),ones(N,1),zeros(N,1),ones(N,1),zeros(N,4);
    zeros(N,3*K),ones(N,1),zeros(N,6)];
H2 = [zeros(N,3*K),ones(N,1),zeros(N,1),ones(N,1),zeros(N,4);
    zeros(N,3*K+7)];
H3 = [zeros(N,3*K+1),ones(N,1),zeros(N,1),ones(N,1),zeros(N,3);
    zeros(N,3*K+7)];

for j=1:N
    datau(j,:) = (uo-so(:,j))'/norm(uo-so(:,j));
    datat(j,:) = (to-so(:,j))'/norm(to-so(:,j));
end
dataut = (uo-to)'/norm(uo-to);
datatu = -dataut;

for j=1:N
    J1_1(j,(1+(j-1)*K):(K*j)) = datau(j,:);
    J1_2(j,(1+(j-1)*K):(K*j)) = datat(j,:);
    J3_1(j,(1+(j-1)*K):(K*j)) = uo_v'/norm(uo-so(:,j))- ...
        uo_v'*(uo-so(:,j))/(norm(uo-so(:,j)))^2*datau(j,:);
end
J1 = blkdiag(J1_1,J1_2);
J2 = blkdiag(J1_1,zeros(N,3*N));
J3 = blkdiag(J3_1,zeros(N,3*N));

for j=1:N
    V1(j,:) = (to-so(:,j))'+(ro(j)-delta_o)*dataut;
    V2(j,:) = (uo-to)'-(ro(j)-delta_o)*dataut;
    V3(j,:) = (ro(j)-delta_o)-norm(uo-to);
    V4(j,:) = (to-so(:,j))';
    V5(j,:) = do(j)-delta_o;
    V6(j,:) = (ro(j)-delta_o)*(uo_v'/norm(uo-to)-(uo-to)'*uo_v/(norm(uo-to))^2*dataut)+(ro_v(j)-delta_fo)*dataut;
    V7(j,:) = (ro(j)-delta_o)*dataut+(to-so(:,j))';
    V8(j,:) = uo_v'+(delta_fo-ro_v(j))*dataut+(delta_o-ro(j))*(uo_v'/norm(uo-to)-(uo-to)'*uo_v/(norm(uo-to))^2*dataut);
    V9(j,:) = (ro_v(j)-delta_fo)-dataut*uo_v;
    V10(j,:) = (ro(j)-delta_o)-norm(uo-to);
    V11(j,:) = 1;
end

V = [V1,zeros(N,K),V2,V3,zeros(N,1);
    zeros(N,2*K),V4,V5,zeros(N,1);
    V6,V7,V8,V9,V10;
    zeros(N,3*K+1),V11];

To = [eye(K,K),zeros(K,K),zeros(K,K),zeros(K,1),zeros(K,1);
    zeros(K,K),eye(K,K),zeros(K,K),zeros(K,1),zeros(K,1);
    zeros(K,K),zeros(K,K),eye(K,K),zeros(K,1),zeros(K,1);
    zeros(1,K),zeros(1,K),zeros(1,K),1,0;
    zeros(1,K),zeros(1,K),zeros(1,K),0,1;
    dataut,zeros(1,K),datatu,zeros(1,2);
    uo_v'/norm(uo-to)-(uo-to)'*uo_v/norm(uo-to)^2*dataut,dataut,-uo_v'/norm(uo-to)+(uo-to)'*uo_v/norm(uo-to)^2*dataut,zeros(1,2);
    -2*to'+2*dataut*delta_o,zeros(1,K),2*to'-2*uo'-2*dataut*delta_o,2*delta_o+2*norm(uo-to),0;
    zeros(1,2*K),-2*to',2*delta_o,0;
    2*delta_fo*dataut+2*delta_o*(uo_v'/norm(uo-to)-(uo-to)'*uo_v/norm(uo-to)^2*dataut),2*delta_o*dataut-2*to',-(2*delta_fo*dataut+2*delta_o*(uo_v'/norm(uo-to)-(uo-to)'*uo_v/norm(uo-to)^2*dataut))-2*uo_v',2*delta_fo+2*dataut*uo_v,2*delta_o+2*norm(uo-to)];

Ko = Ao*To;
Uo = Ko'*Wo*Ko;
L = (inv(V'*Wo*V))*V'*Wo*Bo;

D1 = blkdiag(kron(eye(N),L(1:K,:)),kron(eye(N),L(2*K+1:3*K,:)));
D2 = blkdiag(kron(eye(N),L(1:K,:)),zeros(K*N,4*N*N));
D3 = blkdiag(kron(eye(N),L(K+1:2*K,:)),zeros(K*N,4*N*N));

Omega11 = [-H1,g1]'*(Q(1:2*N,1:2*N).*Wo(1:2*N,1:2*N))*[-H1,g1];
Omega12 = [-H3,g2]'*(Q(1:2*N,1:2*N).*Wo(2*N+1:4*N,1:2*N))*[-H1,g1];
Omega13 = [-H1,g1]'*(Q(1:2*N,1:2*N).*Wo(1:2*N,2*N+1:4*N))*[-H3,g2];
Omega14 = [-H3,g2]'*(Q(1:2*N,1:2*N).*Wo(2*N+1:4*N,2*N+1:4*N))*[-H3,g2]+ ...
    [-H2,g3]'*(Q(2*N+1:4*N,2*N+1:4*N).*Wo(2*N+1:4*N,2*N+1:4*N))*[-H2,g3];
Omega1 = Omega11+Omega12+Omega13+Omega14;

inv_b = inv(Bo');
Omega21 = [-H1,g1]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,1:2*N),ones(1,4*N))))*D1'*J1';
Omega22 = [-H3,g2]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*D1'*J1'+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*D1'*J1';
Omega23 = [-H1,g1]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2');
Omega24 = [-H3,g2]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2')+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,1:2*N),ones(1,4*N))))*(D2'*J3' + D3'*J2');
Omega25 = [-H1,g1]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(1:2*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2';
Omega26 = [-H3,g2]'*((kron(ones(1,2*N),Q(1:2*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2'+ ...
    [-H2,g3]'*((kron(ones(1,2*N),Q(2*N+1:4*N,1:4*N))).*(kron(inv_b(2*N+1:4*N,2*N+1:4*N),ones(1,4*N))))*D2'*J2';
Omega2 = [Omega21+Omega22,Omega23+Omega24+Omega25+Omega26]*Wo*G;

inv_b1 = inv(Bo);
Omega31 = J1*D1*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega32 = J1*D1*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    J1*D1*((kron(ones(2*N,1),Q(1:4*N,2*N+1:4*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega33 = (J2*D3+J3*D2)*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega34 = (J3*D2+J2*D3)*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    (J3*D2+J2*D3)*((kron(ones(2*N,1),Q(1:4*N,2*N+1:4*N))).*(kron(inv_b1(1:2*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega35 = J2*D2*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(2*N+1:4*N,1:2*N),ones(4*N,1))))*[-H1,g1];
Omega36 = J2*D2*((kron(ones(2*N,1),Q(1:4*N,1:2*N))).*(kron(inv_b1(2*N+1:4*N,2*N+1:4*N),ones(4*N,1))))*[-H3,g2]+ ...
    J2*D2*((kron(ones(2*N,1),Q(1:4*N,2*N+1:4*N))).*(kron(inv_b1(2*N+1:4*N,2*N+1:4*N),ones(4*N,1))))*[-H2,g3];
Omega3 = G'*Wo*[Omega31+Omega32;Omega33+Omega34+Omega35+Omega36];

a411 = Wo(1:2*N,1:2*N)*J1*D1;
a421 = Wo(2*N+1:4*N,1:2*N)*J1*D1;
a431 = Wo(1:2*N,2*N+1:4*N)*J2*D3;
a434 = Wo(1:2*N,2*N+1:4*N)*J3*D2;
a441 = Wo(2*N+1:4*N,2*N+1:4*N)*J2*D3;
a442 = Wo(2*N+1:4*N,2*N+1:4*N)*J3*D2;
a451 = Wo(1:2*N,2*N+1:4*N)*J2*D2;
a461 = Wo(2*N+1:4*N,2*N+1:4*N)*J2*D2;
for i=1:2*N
    a412(:,i) = diag(a411(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a422(:,i) = diag(a421(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a423(:,i) = diag(a421(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,2*N+1:4*N));
    a432(:,i) = diag(a431(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a433(:,i) = diag(a434(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a443(:,i) = diag(a441(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a444(:,i) = diag(a442(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a445(:,i) = diag(a441(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,2*N+1:4*N));
    a446(:,i) = diag(a442(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,2*N+1:4*N));
    a452(:,i) = diag(a451(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a462(:,i) = diag(a461(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,1:2*N));
    a463(:,i) = diag(a461(:,4*N*(i-1)+1:4*N*i)*Q(1:4*N,2*N+1:4*N));
end
Omega41 = [-H1,g1]'*a412;
Omega42 = [-H3,g2]'*a422+[-H2,g3]'*a423;
Omega43 = [-H1,g1]'*(a432+a433);
Omega44 = [-H3,g2]'*(a443+a444)+[-H2,g3]'*(a445+a446);
Omega45 = [-H1,g1]'*a452;
Omega46 = [-H3,g2]'*a462+[-H2,g3]'*a463;
Omega4 = [Omega41+Omega42+Omega43+Omega44,Omega45+Omega46]*(inv(Bo))*G;

a511 = D1'*J1'*Wo(1:2*N,1:2*N);
a521 = D3'*J2'*Wo(2*N+1:4*N,1:2*N);
a522 = D2'*J3'*Wo(2*N+1:4*N,1:2*N);
a531 = D1'*J1'*Wo(1:2*N,2*N+1:4*N);
a541 = D3'*J2'*Wo(2*N+1:4*N,2*N+1:4*N);
a542 = D2'*J3'*Wo(2*N+1:4*N,2*N+1:4*N);
a551 = D2'*J2'*Wo(2*N+1:4*N,1:2*N);
a561 = D2'*J2'*Wo(2*N+1:4*N,2*N+1:4*N);
for i=1:2*N
    a512(i,:) = diag(Q(1:2*N,1:4*N)*a511(4*N*(i-1)+1:4*N*i,:));
    a523(i,:) = diag(Q(1:2*N,1:4*N)*a521(4*N*(i-1)+1:4*N*i,:));
    a524(i,:) = diag(Q(1:2*N,1:4*N)*a522(4*N*(i-1)+1:4*N*i,:));
    a532(i,:) = diag(Q(1:2*N,1:4*N)*a531(4*N*(i-1)+1:4*N*i,:));
    a533(i,:) = diag(Q(2*N+1:4*N,1:4*N)*a531(4*N*(i-1)+1:4*N*i,:));
    a543(i,:) = diag(Q(1:2*N,1:4*N)*a541(4*N*(i-1)+1:4*N*i,:));
    a544(i,:) = diag(Q(2*N+1:4*N,1:4*N)*a541(4*N*(i-1)+1:4*N*i,:));
    a545(i,:) = diag(Q(1:2*N,1:4*N)*a542(4*N*(i-1)+1:4*N*i,:));
    a546(i,:) = diag(Q(2*N+1:4*N,1:4*N)*a542(4*N*(i-1)+1:4*N*i,:));
    a552(i,:) = diag(Q(1:2*N,1:4*N)*a551(4*N*(i-1)+1:4*N*i,:));
    a562(i,:) = diag(Q(1:2*N,1:4*N)*a561(4*N*(i-1)+1:4*N*i,:));
    a563(i,:) = diag(Q(2*N+1:4*N,1:4*N)*a561(4*N*(i-1)+1:4*N*i,:));
end

Omega51 = a512*[-H1,g1];
Omega52 = (a523+a524)*[-H1,g1];
Omega53 = a532*[-H3,g2]+a533*[-H2,g3];
Omega54 = (a543+a545)*[-H3,g2]+(a544+a546)*[-H2,g3];
Omega55 = a552*[-H1,g1];
Omega56 = a562*[-H3,g2]+a563*[-H2,g3];
Omega5 = G'*(inv(Bo'))*[Omega51+Omega52+Omega53+Omega54;Omega55+Omega56];

Omega = Omega1-Omega2-Omega3-Omega4-Omega5;
% Omega = Omega1;

Omega111 = Omega(1:(3*K+7),1:(3*K+7));
Omega112 = Omega(1:(3*K+7),(3*K+8));
% Omega21 = Omega((N+1):2*N,1:N);
Omega122 = Omega((3*K+8),(3*K+8));

% zo=[uo',uo_v',to',delta_o,delta_fo]';

fi1 = delta_o^2-2*to'*uo+norm(to)^2+2*delta_o*norm(uo-to);
fi1_d = 2*delta_o*delta_fo-2*to'*uo_v+2*delta_fo*norm(uo-to)+2*delta_o*(uo-to)'/norm(uo-to)*uo_v;
fi2 = delta_o^2-norm(to)^2;

x = [uo',uo_v',to',delta_o,delta_fo,norm(uo-to), ...
    (uo-to)'/norm(uo-to)*uo_v,fi1,fi2,fi1_d]';

v  = 2*(To'*Omega112+To'*Omega111*x);
eta = Omega122+2*Omega112'*x+x'*Omega111*x;
mu = 1/eta;

P1 = Wo*Bo;
P2 = Wo*Ko*(inv(Uo))*Ko'*Wo*Bo;
P3 = To*(inv(Uo))*Ko'*Wo*Bo;
P4 = (inv(Bo))*Ko*(inv(Uo))*Ko'*Wo*Bo;
%
a111 = D1'*J1'*P1(1:2*N,:);
a121 = D3'*J2'*P1(2*N+1:4*N,:);
a131 = D2'*J3'*P1(2*N+1:4*N,:);
a141 = D2'*J2'*P1(2*N+1:4*N,:);
%
b111 = D1'*J1'*P2(1:2*N,:);
b121 = D3'*J2'*P2(2*N+1:4*N,:);
b131 = D2'*J3'*P2(2*N+1:4*N,:);
b141 = D2'*J2'*P2(2*N+1:4*N,:);
%
c1 = reshape((P4(1:2*N,:)*Q(1:4*N,1:4*N))',8*N*N,1);
c2 = reshape((P4(2*N+1:4*N,:)*Q(1:4*N,1:4*N))',8*N*N,1);
%
for j=1:2*N
    a112(j) = trace(Q(1:4*N,1:4*N)*a111(4*N*(j-1)+1:4*N*j,:));
    a122(j) = trace(Q(1:4*N,1:4*N)*a121(4*N*(j-1)+1:4*N*j,:));
    a132(j) = trace(Q(1:4*N,1:4*N)*a131(4*N*(j-1)+1:4*N*j,:));
    a142(j) = trace(Q(1:4*N,1:4*N)*a141(4*N*(j-1)+1:4*N*j,:));
    %
    b112(j) = trace(Q(1:4*N,1:4*N)*b111(4*N*(j-1)+1:4*N*j,:));
    b122(j) = trace(Q(1:4*N,1:4*N)*b121(4*N*(j-1)+1:4*N*j,:));
    b132(j) = trace(Q(1:4*N,1:4*N)*b131(4*N*(j-1)+1:4*N*j,:));
    b142(j) = trace(Q(1:4*N,1:4*N)*b141(4*N*(j-1)+1:4*N*j,:));
end
% %---------------------------------h-------------------------------------%
dataxitheta = [1/norm(uo-to)*eye(K)-((dataut'/norm(uo-to))*dataut), zeros(K,K), -1/norm(uo-to)*eye(K)+((dataut'/norm(uo-to))*dataut),zeros(K,2);
    zeros(K,3*K+2);
    -1/norm(uo-to)*eye(K)+((dataut'/norm(uo-to))*dataut),zeros(K,K), 1/norm(uo-to)*eye(K)-((dataut'/norm(uo-to))*dataut),zeros(K,2);
    zeros(2,3*K+2)];
datadotxitheta = [-2*(dataut'*(uo_v'/norm(uo-to)^2))-(((uo-to)'*uo_v)/norm(uo-to)^3*eye(K))+3*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut, 1/norm(uo-to)*eye(K)-((dataut'/norm(uo-to))*dataut), 2*((uo_v/norm(uo-to)^2)*dataut)+(((uo-to)'*uo_v)/norm(uo-to)^3*eye(K))-3*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut, zeros(K,2);
    1/norm(uo-to)*eye(K)-((dataut'/norm(uo-to))*dataut), zeros(K,K), -1/norm(uo-to)*eye(K)+((dataut'/norm(uo-to))*dataut),zeros(K,2);
    2*(dataut'*(uo_v'/norm(uo-to)^2))+(((uo-to)'*uo_v)/norm(uo-to)^3*eye(K))-3*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut, -1/norm(uo-to)*eye(K)+((dataut'/norm(uo-to))*dataut), -2*(dataut'*(uo_v'/norm(uo-to)^2))-(((uo-to)'*uo_v)/norm(uo-to)^3*eye(K))+3*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut, zeros(K,2);
    zeros(2,3*K+2)];
dataphi1theta = 2*[delta_o/norm(uo-to)*eye(K)-(delta_o*dataut'/norm(uo-to)*dataut),zeros(K,K),-delta_o/norm(uo-to)*eye(K)-eye(K)+(delta_o*dataut'/norm(uo-to)*dataut),2*dataut',zeros(K,1);
    zeros(K,2*K),zeros(K,K),zeros(K,2);
    -delta_o/norm(uo-to)*eye(K)-eye(K)+(delta_o*dataut'/norm(uo-to)*dataut),zeros(K,K),delta_o/norm(uo-to)*eye(K)+eye(K)-(delta_o*dataut'/norm(uo-to)*dataut),-2*dataut',zeros(K,1);
    zeros(1,3*K),1,0;
    zeros(1,3*K+2)];
dataphi2theta = 2*[zeros(2*K,3*K+2);
    zeros(K,2*K),-eye(K),zeros(K,2);
    zeros(1,3*K),1,0;
    zeros(1,3*K+2)];
datadotphi1theta = 2*[delta_fo/norm(uo-to)*eye(K)-(delta_fo*dataut'/norm(uo-to)*dataut)-2*(delta_o*dataut'*(uo_v'/norm(uo-to)^2))+3*delta_o*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut-delta_o*dataut*uo_v/norm(uo-to)^2*eye(K), zeros(K,K), 2*((delta_o*uo_v/norm(uo-to)^2)*dataut)-3*delta_o*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut+delta_o*dataut*uo_v/norm(uo-to)^2*eye(K)-delta_fo/norm(uo-to)*eye(K)+(delta_fo*dataut'/norm(uo-to)*dataut), 2*uo_v/norm(uo-to),2*dataut';
    2*delta_o/norm(uo-to)*eye(K)-2*delta_o*dataut'*dataut/norm(uo-to), zeros(K,K), -eye(K)-2*delta_o/norm(uo-to)*eye(K)+2*delta_o*dataut'*dataut/norm(uo-to),2*dataut',zeros(K,1);
    2*(delta_o*dataut'*(uo_v'/norm(uo-to)^2))-3*delta_o*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut+delta_o*dataut*uo_v/norm(uo-to)^2*eye(K)-delta_fo/norm(uo-to)*eye(K)+(delta_fo*dataut'/norm(uo-to)*dataut), -eye(K), -2*(delta_o*dataut'*(uo_v'/norm(uo-to)^2))+3*delta_o*dataut'*uo_v'/norm(uo-to)^2*dataut'*dataut-delta_o*dataut*uo_v/norm(uo-to)^2*eye(K)+delta_fo/norm(uo-to)*eye(K)-(delta_fo*dataut'/norm(uo-to)*dataut), -2*uo_v/norm(uo-to),-2*dataut';
    -2*dataut*uo_v*dataut/norm(uo-to),zeros(1,K),2*dataut*uo_v*dataut/norm(uo-to),0,1;
    zeros(1,3*K),1,0];
h_o = 0.5*[zeros(3*K+2,1);
    trace(dataxitheta*crlb);
    trace(datadotxitheta*crlb);
    trace(dataphi1theta*crlb);
    trace(dataphi2theta*crlb);
    trace(datadotphi1theta*crlb)];
Btheta = (Ao'*Wo*Bo);
dottheta1 = dataxitheta'*L*(Btheta(3*K+3,:)*Q)'+datadotxitheta'*L*(Btheta(3*K+4,:)*Q)'+dataphi1theta'*L*(Btheta(3*K+5,:)*Q)'+dataphi2theta'*L*(Btheta(3*K+6,:)*Q)'+datadotphi1theta'*L*(Btheta(3*K+7,:)*Q)';
Bfi = (Ao'*Wo*Ao*To*L);
dottheta2 = dataxitheta'*L*(Bfi(3*K+3,:)*Q)'+datadotxitheta'*L*(Bfi(3*K+4,:)*Q)'+dataphi1theta'*L*(Bfi(3*K+5,:)*Q)'+dataphi2theta'*L*(Bfi(3*K+6,:)*Q)'+datadotphi1theta'*L*(Bfi(3*K+7,:)*Q)';
% %----------------------------------------------------------------------------------

Z_star_BS = -(inv(Uo))*To'*(H1'*diag(P2(1:2*N,:)*Q(1:4*N,1:2*N))+H3'*diag(P2(2*N+1:4*N,:)*Q(1:4*N,1:2*N))+H2'*diag(P2(2*N+1:4*N,:)*Q(1:4*N,2*N+1:4*N))) ...
    +(inv(Uo))*To'*(H1'*diag(P1(1:2*N,:)*Q(1:4*N,1:2*N))+H3'*diag(P1(2*N+1:4*N,:)*Q(1:4*N,1:2*N))+H2'*diag(P1(2*N+1:4*N,:)*Q(1:4*N,2*N+1:4*N))) ...
    +(inv(Uo))*Ko'*(inv(Bo'))*[b112'+(b122'+b132');b142'] ...
    +(inv(Uo))*Ko'*Wo*[J1*D1*c1;(J3*D2+J2*D3)*c1+J2*D2*c2] ...
    -((inv(Uo))*Ko'*Wo*[diag((H1*P3)*Q(1:4*N,1:2*N));diag((H2*P3)*Q(1:4*N,2*N+1:4*N))+diag((H3*P3)*Q(1:4*N,1:2*N))]) ...
    -mu*(inv(Uo))*v*trace(Bo'*Wo*Ko*(inv(Uo))*Ko'*Wo*Bo*Q) ...
    -mu*(inv(Uo))*v ...
    -(inv(Uo))*Ko'*(inv(Bo'))*[a112'+(a122'+a132');a142'] ...
    -0.5*(inv(Uo))*Ko'*Wo*[diag(Q(1:2*N,1:2*N))',2*diag(Q(2*N+1:3*N,1:N))',zeros(N,1)']' ...
    +0.5*mu*(inv(Uo))*v*trace(Bo'*Wo*Bo*Q) ...
    +mu*(inv(Uo))*v ...
    +0.5*mu*(inv(Uo))*v*trace(Uo*(inv(Uo))) ...
    -(inv(Uo))*Ko'*Wo*Ao*h_o ...
    +(inv(Uo))*dottheta1-(inv(Uo))*dottheta2;
end
