function MSE=MSLocUknTxPosMvgObj_CRLB(to,uo,uo_v,S,Q)
% function MSE=MSLocUknTxPosMvgObj_CRLB(to,uo,uo_v,S,Q)
% 
% This function computes the CRLB of the joint estimation of the object position and 
% velocity, the transmitter position, and the range and range rate offsets.
% 
% Usage: mse(:,:,iii) = MSLocUknTxPosMvgObj_CRLB(to(:,iii),uo(:,iii),uo_v,so(:,1:N,iii),Q);
% 
% Input Parameter List:
% to:   (KxM)xcj transmitter position matrix.
% uo:   Kxcj object position matrix.
% uo_v: Kxcj object velocity matrix.
% so:   (KxN)xcj receiver positions matrix.
%       N: Number of receivers.
% Q:    Covariance matrix
%
% Output Parameter List:
% mse:  (3*K+2)x(3*K+2), CRLB matrix of the object position and velocity,
%       the transmitter position, and the range and range rate offsets estimate.      
% 
% Reference:
% J. Pei, G. Wang, K. C. Ho, and L. Huang, "Reducing bias for multistatic 
% localization of a moving object by transmitter at unknown position," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 5, pp. 5324-5341, Oct. 2023.
% 
% Jian Pei, Gang Wang, K. C. Ho and Lei Huang	Nov. 1, 2023
%
%       Copyright (C) 2023
%       Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China.
%       wanggang@nbu.edu.cn
%

[K,N]=size(S);

for j=1:N
    delta_ru(j,:)=(uo-S(:,j))'/norm(uo-S(:,j))+(uo-to)'/norm(uo-to);
    delta_rt(j,:)=-(uo-to)'/norm(uo-to);
    delta_rdu(j,:)=uo_v/norm(uo-S(:,j))-((uo-S(:,j))'*uo_v*(uo-S(:,j)))/(norm(uo-S(:,j))^3)...
                   +(uo_v)/norm(uo-to)-((uo-to)'*(uo_v)*(uo-to))/(norm(uo-to)^3); 
    delta_rdt(j,:)=(-uo_v)/norm(uo-to)+((to-uo)'*(uo_v)*(to-uo))/(norm(uo-to)^3);
    delta_dt(j,:)=(to-S(:,j))'/norm(to-S(:,j));     
end

delta_rdud=delta_ru;

RR=[delta_ru,zeros(N,K),delta_rt,ones(N,1),zeros(N,1);
    zeros(N,K),zeros(N,K),delta_dt,ones(N,1),zeros(N,1);
    delta_rdu,delta_rdud,delta_rdt,zeros(N,1),ones(N,1);
    zeros(N,K),zeros(N,K),zeros(N,K),zeros(N,1),ones(N,1)];

MSE=inv(RR'*inv(Q)*RR);

end
