function Y = MSLocUknTxPosMvgObj_SDP(r,d,r_v,d_v,so,Q1)
% function Y = MSLocUknTxPosMvgObj_SDP(r,d,r_v,d_v,so,Q1)
% 
% This program realizes the SDP solution for estimate the object position and velocity, the 
% transmitter position, and the range and range rate offsets using the TD and DFS measurement.
%
% Usage: Y = MSLocUknTxPosMvgObj_SDP(r,d,r_v,d_v,s,Q1);
% 
% Input Parameter List:
% r:   Nx1 Indirect range measurement vector after narrow scene.
% d:   Nx1 Direct range measurement vector after narrow scene.
% r_v: Nx1 Indirect range rate measurement vector after narrow scene.
% d_v: Nx1 Direct range rate measurement vector after narrow scene.
% s:   KxN Receiver position matrix after narrow scene.
% Q1:  (4*N)x(4*N) Covariance matrix after narrow scene.
%
% Output Parameter List:
% Y:   (3*K+8)x(3*K+8) The solution of SDP problem.
% 
% Reference:
% J. Pei, G. Wang, K. C. Ho, and L. Huang, "Reducing bias for multistatic
% localization of a moving object by transmitter at unknown position," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 5, pp. 5324-5341, Oct. 2023.
% 
% Jian Pei, Gang Wang, K. C. Ho and Lei Huang
%
%       Copyright (C) 2023
%       Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China.
%       wanggang@nbu.edu.cn.
%


[K,N] = size(so);

for j=1:N
    b1(j) = 0.5*(r(j)^2-norm(so(:,j))^2);
    b2(j) = 0.5*(d(j)^2-norm(so(:,j))^2);
    b3(j) = r(j)*r_v(j);
    b4(j) = d_v(j);
end
b = [b1,b2,b3,b4]'; %the matrix 'b' in (15)

A1 = [-so',zeros(N,2*K),r,zeros(N,1),r,zeros(N,1),-0.5*ones(N,1),zeros(N,2)];
A2 = [zeros(N,2*K),-so',d,zeros(N,4),-0.5*ones(N,1),zeros(N,1)];
A3 = [zeros(N,K),-so',zeros(N,K),r_v,r,r_v,r,zeros(N,2),-0.5*ones(N,1)];
A4 = [zeros(N,3*K+1),ones(N,1),zeros(N,5)];
A = [A1',A2',A3',A4']'; %the matrix 'A' in (15)

G = [-A,b]; %the matrix 'G' in (18a)

W = (inv(Q1)); 

for i=1:2 % i=1, to obtain an inital estimate; i=2 using the initial estimate to solve SDP problem again
    cvx_clear
    cvx_begin sdp
    cvx_solver sdpt3
%     cvx_solver sedumi
    cvx_precision best
    cvx_quiet(1)
    variable Y(3*K+8,3*K+8) symmetric
    minimize (trace(G'*W*G*Y))
    subject to

        Y(3*K+3,3*K+3) == trace(Y(1:K,1:K))+trace(Y(2*K+1:3*K,2*K+1:3*K))-2*trace(Y(1:K,(2*K+1:3*K)));
        Y(3*K+3,3*K+4) == trace(Y(1:K,K+1:2*K))-trace(Y(2*K+1:3*K,K+1:2*K));
        Y(3*K+5,3*K+8) == Y(3*K+1,3*K+1)+2*Y(3*K+1,3*K+3)-2*trace(Y(1:K,2*K+1:3*K))+trace(Y(2*K+1:3*K,2*K+1:3*K));
        Y(3*K+6,3*K+8) == Y(3*K+1,3*K+1)-trace(Y(2*K+1:3*K,2*K+1:3*K));
        Y(3*K+7,3*K+8) == 2*Y(3*K+1,3*K+2)+2*Y(3*K+1,3*K+4)+2*Y(3*K+2,3*K+3)-2*trace(Y(2*K+1:3*K,K+1:2*K));
        
        Y(3*K+4,3*K+4) <= trace(Y(K+1:2*K,K+1:2*K));
       
        Y(3*K+8,3*K+8) == 1;
        
        Y >= 0;  
        
    cvx_end
      
    %====update W====
    u_hat = Y(1:K,3*K+8)/Y(3*K+8,3*K+8);
    uv_hat = Y(K+1:2*K,3*K+8)/Y(3*K+8,3*K+8);
    t_hat = Y(2*K+1:3*K,3*K+8)/Y(3*K+8,3*K+8);
    
    for j=1:N
        b11(j) = norm(u_hat - so(:,j));
        b21(j) = norm(t_hat - so(:,j));
        b31(j) = ((u_hat - so(:,j))'/norm(u_hat - so(:,j)))*uv_hat;
        b41(j) = 1;
    end
    B1 = diag(b11);
    B2 = diag(b21);
    B3 = diag(b31);
    B4 = diag(b41);

    B = [blkdiag(B1,B2),zeros(2*N,2*N);
         blkdiag(B3,zeros(N,N)),blkdiag(B1,B4)];

    W = (inv(B*Q1*B'));

end
end
                  